package com.xxl.job.admin.service.impl;

package .var.folders.cl.9twcgjfx4vsg5zl73c6wxmvm0000gn.T;

import com.xxl.job.admin.core.cron.CronExpression;
import com.xxl.job.admin.core.model.XxlJobGroup;
import com.xxl.job.admin.core.model.XxlJobInfo;
import com.xxl.job.admin.core.model.XxlJobLogReport;
import com.xxl.job.admin.core.model.XxlJobUser;
import com.xxl.job.admin.core.route.ExecutorRouteStrategyEnum;
import com.xxl.job.admin.core.scheduler.MisfireStrategyEnum;
import com.xxl.job.admin.core.scheduler.ScheduleTypeEnum;
import com.xxl.job.admin.core.thread.JobScheduleHelper;
import com.xxl.job.admin.core.thread.JobTriggerPoolHelper;
import com.xxl.job.admin.core.trigger.TriggerTypeEnum;
import com.xxl.job.admin.core.util.I18nUtil;
import com.xxl.job.admin.dao.XxlJobGroupDao;
import com.xxl.job.admin.dao.XxlJobInfoDao;
import com.xxl.job.admin.dao.XxlJobLogDao;
import com.xxl.job.admin.dao.XxlJobLogGlueDao;
import com.xxl.job.admin.dao.XxlJobLogReportDao;
import com.xxl.job.admin.service.XxlJobService;
import com.xxl.job.core.biz.model.ReturnT;
import com.xxl.job.core.enums.ExecutorBlockStrategyEnum;
import com.xxl.job.core.glue.GlueTypeEnum;
import com.xxl.job.core.util.DateUtil;
import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import javax.annotation.Resource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.stereotype.Service;

@Service
public class XxlJobServiceImpl implements XxlJobService {
  private static Logger logger = LoggerFactory.getLogger(com.xxl.job.admin.service.impl.XxlJobServiceImpl.class);
  
  @Resource
  private XxlJobGroupDao xxlJobGroupDao;
  
  @Resource
  private XxlJobInfoDao xxlJobInfoDao;
  
  @Resource
  public XxlJobLogDao xxlJobLogDao;
  
  @Resource
  private XxlJobLogGlueDao xxlJobLogGlueDao;
  
  @Resource
  private XxlJobLogReportDao xxlJobLogReportDao;
  
  public Map<String, Object> pageList(int start, int length, int jobGroup, int triggerStatus, String jobDesc, String executorHandler, String author) {
    List<XxlJobInfo> list = this.xxlJobInfoDao.pageList(start, length, jobGroup, triggerStatus, jobDesc, executorHandler, author);
    int list_count = this.xxlJobInfoDao.pageListCount(start, length, jobGroup, triggerStatus, jobDesc, executorHandler, author);
    Map<String, Object> maps = new HashMap<>();
    maps.put("recordsTotal", Integer.valueOf(list_count));
    maps.put("recordsFiltered", Integer.valueOf(list_count));
    maps.put("data", list);
    return maps;
  }
  
  public ReturnT<String> add(XxlJobInfo jobInfo, XxlJobUser loginUser) {
    XxlJobGroup group = this.xxlJobGroupDao.load(jobInfo.getJobGroup());
    if (group == null)
      return new ReturnT(500, I18nUtil.getString("system_please_choose") + I18nUtil.getString("jobinfo_field_jobgroup")); 
    if (jobInfo.getJobDesc() == null || jobInfo.getJobDesc().trim().length() == 0)
      return new ReturnT(500, I18nUtil.getString("system_please_input") + I18nUtil.getString("jobinfo_field_jobdesc")); 
    if (jobInfo.getAuthor() == null || jobInfo.getAuthor().trim().length() == 0)
      return new ReturnT(500, I18nUtil.getString("system_please_input") + I18nUtil.getString("jobinfo_field_author")); 
    ScheduleTypeEnum scheduleTypeEnum = ScheduleTypeEnum.match(jobInfo.getScheduleType(), null);
    if (scheduleTypeEnum == null)
      return new ReturnT(500, I18nUtil.getString("schedule_type") + I18nUtil.getString("system_unvalid")); 
    if (scheduleTypeEnum == ScheduleTypeEnum.CRON) {
      if (jobInfo.getScheduleConf() == null || !CronExpression.isValidExpression(jobInfo.getScheduleConf()))
        return new ReturnT(500, "Cron" + I18nUtil.getString("system_unvalid")); 
    } else if (scheduleTypeEnum == ScheduleTypeEnum.FIX_RATE) {
      if (jobInfo.getScheduleConf() == null)
        return new ReturnT(500, I18nUtil.getString("schedule_type")); 
      try {
        int fixSecond = Integer.valueOf(jobInfo.getScheduleConf()).intValue();
        if (fixSecond < 1)
          return new ReturnT(500, I18nUtil.getString("schedule_type") + I18nUtil.getString("system_unvalid")); 
      } catch (Exception e) {
        return new ReturnT(500, I18nUtil.getString("schedule_type") + I18nUtil.getString("system_unvalid"));
      } 
    } 
    if (GlueTypeEnum.match(jobInfo.getGlueType()) == null)
      return new ReturnT(500, I18nUtil.getString("jobinfo_field_gluetype") + I18nUtil.getString("system_unvalid")); 
    if (GlueTypeEnum.BEAN == GlueTypeEnum.match(jobInfo.getGlueType()) && (jobInfo.getExecutorHandler() == null || jobInfo.getExecutorHandler().trim().length() == 0))
      return new ReturnT(500, I18nUtil.getString("system_please_input") + "JobHandler"); 
    if (GlueTypeEnum.GLUE_SHELL == GlueTypeEnum.match(jobInfo.getGlueType()) && jobInfo.getGlueSource() != null)
      jobInfo.setGlueSource(jobInfo.getGlueSource().replaceAll("\r", "")); 
    if (ExecutorRouteStrategyEnum.match(jobInfo.getExecutorRouteStrategy(), null) == null)
      return new ReturnT(500, I18nUtil.getString("jobinfo_field_executorRouteStrategy") + I18nUtil.getString("system_unvalid")); 
    if (MisfireStrategyEnum.match(jobInfo.getMisfireStrategy(), null) == null)
      return new ReturnT(500, I18nUtil.getString("misfire_strategy") + I18nUtil.getString("system_unvalid")); 
    if (ExecutorBlockStrategyEnum.match(jobInfo.getExecutorBlockStrategy(), null) == null)
      return new ReturnT(500, I18nUtil.getString("jobinfo_field_executorBlockStrategy") + I18nUtil.getString("system_unvalid")); 
    if (jobInfo.getChildJobId() != null && jobInfo.getChildJobId().trim().length() > 0) {
      String[] childJobIds = jobInfo.getChildJobId().split(",");
      for (String childJobIdItem : childJobIds) {
        if (childJobIdItem != null && childJobIdItem.trim().length() > 0 && isNumeric(childJobIdItem)) {
          XxlJobInfo childJobInfo = this.xxlJobInfoDao.loadById(Integer.parseInt(childJobIdItem));
          if (childJobInfo == null)
            return new ReturnT(500, 
                MessageFormat.format(I18nUtil.getString("jobinfo_field_childJobId") + "({0})" + I18nUtil.getString("system_not_found"), new Object[] { childJobIdItem })); 
          if (!loginUser.validPermission(childJobInfo.getJobGroup()))
            return new ReturnT(500, 
                MessageFormat.format(I18nUtil.getString("jobinfo_field_childJobId") + "({0})" + I18nUtil.getString("system_permission_limit"), new Object[] { childJobIdItem })); 
        } else {
          return new ReturnT(500, 
              MessageFormat.format(I18nUtil.getString("jobinfo_field_childJobId") + "({0})" + I18nUtil.getString("system_unvalid"), new Object[] { childJobIdItem }));
        } 
      } 
      String temp = "";
      for (String item : childJobIds)
        temp = temp + item + ","; 
      temp = temp.substring(0, temp.length() - 1);
      jobInfo.setChildJobId(temp);
    } 
    jobInfo.setAddTime(new Date());
    jobInfo.setUpdateTime(new Date());
    jobInfo.setGlueUpdatetime(new Date());
    this.xxlJobInfoDao.save(jobInfo);
    if (jobInfo.getId() < 1)
      return new ReturnT(500, I18nUtil.getString("jobinfo_field_add") + I18nUtil.getString("system_fail")); 
    return new ReturnT(String.valueOf(jobInfo.getId()));
  }
  
  private boolean isNumeric(String str) {
    try {
      int result = Integer.valueOf(str).intValue();
      return true;
    } catch (NumberFormatException e) {
      return false;
    } 
  }
  
  public ReturnT<String> update(XxlJobInfo jobInfo, XxlJobUser loginUser) {
    if (jobInfo.getJobDesc() == null || jobInfo.getJobDesc().trim().length() == 0)
      return new ReturnT(500, I18nUtil.getString("system_please_input") + I18nUtil.getString("jobinfo_field_jobdesc")); 
    if (jobInfo.getAuthor() == null || jobInfo.getAuthor().trim().length() == 0)
      return new ReturnT(500, I18nUtil.getString("system_please_input") + I18nUtil.getString("jobinfo_field_author")); 
    ScheduleTypeEnum scheduleTypeEnum = ScheduleTypeEnum.match(jobInfo.getScheduleType(), null);
    if (scheduleTypeEnum == null)
      return new ReturnT(500, I18nUtil.getString("schedule_type") + I18nUtil.getString("system_unvalid")); 
    if (scheduleTypeEnum == ScheduleTypeEnum.CRON) {
      if (jobInfo.getScheduleConf() == null || !CronExpression.isValidExpression(jobInfo.getScheduleConf()))
        return new ReturnT(500, "Cron" + I18nUtil.getString("system_unvalid")); 
    } else if (scheduleTypeEnum == ScheduleTypeEnum.FIX_RATE) {
      if (jobInfo.getScheduleConf() == null)
        return new ReturnT(500, I18nUtil.getString("schedule_type") + I18nUtil.getString("system_unvalid")); 
      try {
        int fixSecond = Integer.valueOf(jobInfo.getScheduleConf()).intValue();
        if (fixSecond < 1)
          return new ReturnT(500, I18nUtil.getString("schedule_type") + I18nUtil.getString("system_unvalid")); 
      } catch (Exception e) {
        return new ReturnT(500, I18nUtil.getString("schedule_type") + I18nUtil.getString("system_unvalid"));
      } 
    } 
    if (ExecutorRouteStrategyEnum.match(jobInfo.getExecutorRouteStrategy(), null) == null)
      return new ReturnT(500, I18nUtil.getString("jobinfo_field_executorRouteStrategy") + I18nUtil.getString("system_unvalid")); 
    if (MisfireStrategyEnum.match(jobInfo.getMisfireStrategy(), null) == null)
      return new ReturnT(500, I18nUtil.getString("misfire_strategy") + I18nUtil.getString("system_unvalid")); 
    if (ExecutorBlockStrategyEnum.match(jobInfo.getExecutorBlockStrategy(), null) == null)
      return new ReturnT(500, I18nUtil.getString("jobinfo_field_executorBlockStrategy") + I18nUtil.getString("system_unvalid")); 
    if (jobInfo.getChildJobId() != null && jobInfo.getChildJobId().trim().length() > 0) {
      String[] childJobIds = jobInfo.getChildJobId().split(",");
      for (String childJobIdItem : childJobIds) {
        if (childJobIdItem != null && childJobIdItem.trim().length() > 0 && isNumeric(childJobIdItem)) {
          int childJobId = Integer.parseInt(childJobIdItem);
          if (childJobId == jobInfo.getId())
            return new ReturnT(500, I18nUtil.getString("jobinfo_field_childJobId") + "(" + childJobId + ")" + I18nUtil.getString("system_unvalid")); 
          XxlJobInfo childJobInfo = this.xxlJobInfoDao.loadById(childJobId);
          if (childJobInfo == null)
            return new ReturnT(500, 
                MessageFormat.format(I18nUtil.getString("jobinfo_field_childJobId") + "({0})" + I18nUtil.getString("system_not_found"), new Object[] { childJobIdItem })); 
          if (!loginUser.validPermission(childJobInfo.getJobGroup()))
            return new ReturnT(500, 
                MessageFormat.format(I18nUtil.getString("jobinfo_field_childJobId") + "({0})" + I18nUtil.getString("system_permission_limit"), new Object[] { childJobIdItem })); 
        } else {
          return new ReturnT(500, 
              MessageFormat.format(I18nUtil.getString("jobinfo_field_childJobId") + "({0})" + I18nUtil.getString("system_unvalid"), new Object[] { childJobIdItem }));
        } 
      } 
      String temp = "";
      for (String item : childJobIds)
        temp = temp + item + ","; 
      temp = temp.substring(0, temp.length() - 1);
      jobInfo.setChildJobId(temp);
    } 
    XxlJobGroup jobGroup = this.xxlJobGroupDao.load(jobInfo.getJobGroup());
    if (jobGroup == null)
      return new ReturnT(500, I18nUtil.getString("jobinfo_field_jobgroup") + I18nUtil.getString("system_unvalid")); 
    XxlJobInfo exists_jobInfo = this.xxlJobInfoDao.loadById(jobInfo.getId());
    if (exists_jobInfo == null)
      return new ReturnT(500, I18nUtil.getString("jobinfo_field_id") + I18nUtil.getString("system_not_found")); 
    long nextTriggerTime = exists_jobInfo.getTriggerNextTime();
    boolean scheduleDataNotChanged = (jobInfo.getScheduleType().equals(exists_jobInfo.getScheduleType()) && jobInfo.getScheduleConf().equals(exists_jobInfo.getScheduleConf()));
    if (exists_jobInfo.getTriggerStatus() == 1 && !scheduleDataNotChanged)
      try {
        Date nextValidTime = JobScheduleHelper.generateNextValidTime(jobInfo, new Date(System.currentTimeMillis() + 5000L));
        if (nextValidTime == null)
          return new ReturnT(500, I18nUtil.getString("schedule_type") + I18nUtil.getString("system_unvalid")); 
        nextTriggerTime = nextValidTime.getTime();
      } catch (Exception e) {
        logger.error(e.getMessage(), e);
        return new ReturnT(500, I18nUtil.getString("schedule_type") + I18nUtil.getString("system_unvalid"));
      }  
    exists_jobInfo.setJobGroup(jobInfo.getJobGroup());
    exists_jobInfo.setJobDesc(jobInfo.getJobDesc());
    exists_jobInfo.setAuthor(jobInfo.getAuthor());
    exists_jobInfo.setAlarmEmail(jobInfo.getAlarmEmail());
    exists_jobInfo.setScheduleType(jobInfo.getScheduleType());
    exists_jobInfo.setScheduleConf(jobInfo.getScheduleConf());
    exists_jobInfo.setMisfireStrategy(jobInfo.getMisfireStrategy());
    exists_jobInfo.setExecutorRouteStrategy(jobInfo.getExecutorRouteStrategy());
    exists_jobInfo.setExecutorHandler(jobInfo.getExecutorHandler());
    exists_jobInfo.setExecutorParam(jobInfo.getExecutorParam());
    exists_jobInfo.setExecutorBlockStrategy(jobInfo.getExecutorBlockStrategy());
    exists_jobInfo.setExecutorTimeout(jobInfo.getExecutorTimeout());
    exists_jobInfo.setExecutorFailRetryCount(jobInfo.getExecutorFailRetryCount());
    exists_jobInfo.setChildJobId(jobInfo.getChildJobId());
    exists_jobInfo.setTriggerNextTime(nextTriggerTime);
    exists_jobInfo.setUpdateTime(new Date());
    this.xxlJobInfoDao.update(exists_jobInfo);
    return ReturnT.SUCCESS;
  }
  
  public ReturnT<String> remove(int id) {
    XxlJobInfo xxlJobInfo = this.xxlJobInfoDao.loadById(id);
    if (xxlJobInfo == null)
      return ReturnT.SUCCESS; 
    this.xxlJobInfoDao.delete(id);
    this.xxlJobLogDao.delete(id);
    this.xxlJobLogGlueDao.deleteByJobId(id);
    return ReturnT.SUCCESS;
  }
  
  public ReturnT<String> start(int id) {
    XxlJobInfo xxlJobInfo = this.xxlJobInfoDao.loadById(id);
    ScheduleTypeEnum scheduleTypeEnum = ScheduleTypeEnum.match(xxlJobInfo.getScheduleType(), ScheduleTypeEnum.NONE);
    if (ScheduleTypeEnum.NONE == scheduleTypeEnum)
      return new ReturnT(500, I18nUtil.getString("schedule_type_none_limit_start")); 
    long nextTriggerTime = 0L;
    try {
      Date nextValidTime = JobScheduleHelper.generateNextValidTime(xxlJobInfo, new Date(System.currentTimeMillis() + 5000L));
      if (nextValidTime == null)
        return new ReturnT(500, I18nUtil.getString("schedule_type") + I18nUtil.getString("system_unvalid")); 
      nextTriggerTime = nextValidTime.getTime();
    } catch (Exception e) {
      logger.error(e.getMessage(), e);
      return new ReturnT(500, I18nUtil.getString("schedule_type") + I18nUtil.getString("system_unvalid"));
    } 
    xxlJobInfo.setTriggerStatus(1);
    xxlJobInfo.setTriggerLastTime(0L);
    xxlJobInfo.setTriggerNextTime(nextTriggerTime);
    xxlJobInfo.setUpdateTime(new Date());
    this.xxlJobInfoDao.update(xxlJobInfo);
    return ReturnT.SUCCESS;
  }
  
  public ReturnT<String> stop(int id) {
    XxlJobInfo xxlJobInfo = this.xxlJobInfoDao.loadById(id);
    xxlJobInfo.setTriggerStatus(0);
    xxlJobInfo.setTriggerLastTime(0L);
    xxlJobInfo.setTriggerNextTime(0L);
    xxlJobInfo.setUpdateTime(new Date());
    this.xxlJobInfoDao.update(xxlJobInfo);
    return ReturnT.SUCCESS;
  }
  
  public ReturnT<String> trigger(XxlJobUser loginUser, int jobId, String executorParam, String addressList) {
    if (loginUser == null)
      return new ReturnT(ReturnT.FAIL.getCode(), I18nUtil.getString("system_permission_limit")); 
    XxlJobInfo xxlJobInfo = this.xxlJobInfoDao.loadById(jobId);
    if (xxlJobInfo == null)
      return new ReturnT(ReturnT.FAIL.getCode(), I18nUtil.getString("jobinfo_glue_jobid_unvalid")); 
    if (!hasPermission(loginUser, xxlJobInfo.getJobGroup()))
      return new ReturnT(ReturnT.FAIL.getCode(), I18nUtil.getString("system_permission_limit")); 
    if (executorParam == null)
      executorParam = ""; 
    JobTriggerPoolHelper.trigger(jobId, TriggerTypeEnum.MANUAL, -1, null, executorParam, addressList);
    return ReturnT.SUCCESS;
  }
  
  private boolean hasPermission(XxlJobUser loginUser, int jobGroup) {
    if (loginUser.getRole() == 1)
      return true; 
    List<String> groupIdStrs = new ArrayList<>();
    if (loginUser.getPermission() != null && loginUser.getPermission().trim().length() > 0)
      groupIdStrs = Arrays.asList(loginUser.getPermission().trim().split(",")); 
    return groupIdStrs.contains(String.valueOf(jobGroup));
  }
  
  public Map<String, Object> dashboardInfo() {
    int jobInfoCount = this.xxlJobInfoDao.findAllCount();
    int jobLogCount = 0;
    int jobLogSuccessCount = 0;
    XxlJobLogReport xxlJobLogReport = this.xxlJobLogReportDao.queryLogReportTotal();
    if (xxlJobLogReport != null) {
      jobLogCount = xxlJobLogReport.getRunningCount() + xxlJobLogReport.getSucCount() + xxlJobLogReport.getFailCount();
      jobLogSuccessCount = xxlJobLogReport.getSucCount();
    } 
    Set<String> executorAddressSet = new HashSet<>();
    List<XxlJobGroup> groupList = this.xxlJobGroupDao.findAll();
    if (groupList != null && !groupList.isEmpty())
      for (XxlJobGroup group : groupList) {
        if (group.getRegistryList() != null && !group.getRegistryList().isEmpty())
          executorAddressSet.addAll(group.getRegistryList()); 
      }  
    int executorCount = executorAddressSet.size();
    Map<String, Object> dashboardMap = new HashMap<>();
    dashboardMap.put("jobInfoCount", Integer.valueOf(jobInfoCount));
    dashboardMap.put("jobLogCount", Integer.valueOf(jobLogCount));
    dashboardMap.put("jobLogSuccessCount", Integer.valueOf(jobLogSuccessCount));
    dashboardMap.put("executorCount", Integer.valueOf(executorCount));
    return dashboardMap;
  }
  
  public ReturnT<Map<String, Object>> chartInfo(Date startDate, Date endDate) {
    List<String> triggerDayList = new ArrayList<>();
    List<Integer> triggerDayCountRunningList = new ArrayList<>();
    List<Integer> triggerDayCountSucList = new ArrayList<>();
    List<Integer> triggerDayCountFailList = new ArrayList<>();
    int triggerCountRunningTotal = 0;
    int triggerCountSucTotal = 0;
    int triggerCountFailTotal = 0;
    List<XxlJobLogReport> logReportList = this.xxlJobLogReportDao.queryLogReport(startDate, endDate);
    if (logReportList != null && logReportList.size() > 0) {
      for (XxlJobLogReport item : logReportList) {
        String day = DateUtil.formatDate(item.getTriggerDay());
        int triggerDayCountRunning = item.getRunningCount();
        int triggerDayCountSuc = item.getSucCount();
        int triggerDayCountFail = item.getFailCount();
        triggerDayList.add(day);
        triggerDayCountRunningList.add(Integer.valueOf(triggerDayCountRunning));
        triggerDayCountSucList.add(Integer.valueOf(triggerDayCountSuc));
        triggerDayCountFailList.add(Integer.valueOf(triggerDayCountFail));
        triggerCountRunningTotal += triggerDayCountRunning;
        triggerCountSucTotal += triggerDayCountSuc;
        triggerCountFailTotal += triggerDayCountFail;
      } 
    } else {
      for (int i = -6; i <= 0; i++) {
        triggerDayList.add(DateUtil.formatDate(DateUtil.addDays(new Date(), i)));
        triggerDayCountRunningList.add(Integer.valueOf(0));
        triggerDayCountSucList.add(Integer.valueOf(0));
        triggerDayCountFailList.add(Integer.valueOf(0));
      } 
    } 
    Map<String, Object> result = new HashMap<>();
    result.put("triggerDayList", triggerDayList);
    result.put("triggerDayCountRunningList", triggerDayCountRunningList);
    result.put("triggerDayCountSucList", triggerDayCountSucList);
    result.put("triggerDayCountFailList", triggerDayCountFailList);
    result.put("triggerCountRunningTotal", Integer.valueOf(triggerCountRunningTotal));
    result.put("triggerCountSucTotal", Integer.valueOf(triggerCountSucTotal));
    result.put("triggerCountFailTotal", Integer.valueOf(triggerCountFailTotal));
    return new ReturnT(result);
  }
  
  public XxlJobInfo getById(int id) {
    return this.xxlJobInfoDao.loadById(id);
  }
}
